/***************************************************************************/
/***************************************************************************/
/* QuickC source for TRUNK */
/* command line--> QCL /c /AS /Gs /Ox TRUNK.C */
/*             --> LINK TRUNK.OBJ,TRUNK.EXE,,c:...\SLIBCE.LIB /ST:0x1000 */
/***************************************************************************/
/***************************************************************************/
/**
*** TRUNK -- a replacement for the MS-DOS 'TREE' command.  TREE allows only
*** one option ('/f'--similar to the one implemented in TRUNK), but more
*** to the point, the output is verbose and completely unsuited for passing
*** to other programs.  TRUNK is fully redirectable using MS-DOS redirection
*** and pipes.
***
*** Command line option flags are evaluated in left-to-right order.
*** Flags may conflict with each other, and the flag to the right takes
*** precedence--e.g. the command line 'TRUNK /f /o' will print only a
*** summary, no file names.
***
*** One significant departure from DOS is that TRUNK considers the root to be
*** a directory like any other when summarizing directory structure (obeying
*** the /s or /o flag).  CHKDSK, on the other hand, reports the number of
*** *subdirectories* and leaves the existence of the root directory implied.
*** This difference (more semantic than functional) means that when TRUNK is
*** applied to an entire disk, the number of directories reported found will
*** be one greater than the number of subdirectories reported by CHKDSK.
***
*** Official TRUNK soundtrack:
*** "Fleetwood Mac in Chicago"  Sire: SASH 3175-2 (C) 1975, or
***                             Blue Horizon: BH 3801 (C) 1969
***
*** Enjoy.
**/
/***************************************************************************/
/***************************************************************************/

#include <stdio.h>
#include <dos.h>
#include <sys\types.h>
#include <errno.h>
#include <stdlib.h>
#include <string.h>
#include <direct.h>

#include "trunk.h"

int             dirs_found=0;
int             files_found=0;
unsigned int    srch_attrib=_A_NORMAL | _A_RDONLY | _A_SUBDIR;
int             summary=0;
int             print_dirs=1;
int             print_files=0;
int             print_drive=0;
int             print_path=1;
int             force_lower=0;
int             help=0;
int             info=0;
char            search_drive[]=" :";
char            home_drive[]=" :";



/***************************************************************************/
/***************************************************************************/
/**
*** main
**/
/***************************************************************************/
/***************************************************************************/

void main(int argc, char *argv[])

{
path            current_dir;
unsigned char   *home_cwd_ptr=current_dir;
unsigned char   *alt_cwd_ptr=current_dir;
path            search_dir;
unsigned char   *search_ptr=search_dir;
int             n;



/**
*** Process any command line arguments that are flags.  The last argument
*** is assumed to be the pathname.  Since the pathname is not required, the
*** last argument may be a flag as well.
**/
if (argc > 1) {
	for (n=1; n < argc; n++)    /* argv[0] always == TRUNK */
		{
		if (argv[n][0] == '/')
			switch ( LOCASE(argv[n][1]) ) {
				case 'y':   srch_attrib |= _A_SYSTEM;
				case 'h':   srch_attrib |= _A_HIDDEN;
							break;
				case 'n':   if (argv[n][2] == 'd')
								print_drive=2;
							if (argv[n][2] == 'p')
								print_path=0;
							break;
				case 'o':   print_dirs=0;
							print_files=0;
				case 's':   summary=1;
							break;
				case 'f':   print_dirs=0;
							print_files=1;
							break;
				case 'l':   force_lower=1; break;
				case 'v':   puts(version);
							return;
				case '?':   help=1; break;
				case '$':   info=1; break;
				}
		}   /* for (n=1; n < argc-1; n++) */
	}

/**
*** If help was requested (through the /? flag or by TRUNK ?), print
*** a help message and quit.  Ditto for TRUNK information (/$).
**/
if ( (help) || (argv[1][0] == '?') ) {
	display_strings(help_msg);
	return;
	}
if (info) {
	display_strings(info_msg);
	return;
	}

/**
*** If there are command line arguments, then the last one may be the
*** path to search on.  If the last argument (argv[argc-1]) is not a flag,
*** then it is assumed to be the search path.
***
*** If there is a search path, but it is not a full path but merely a
*** drive specifier (such as from 'TRUNK D:') which is not the current
*** drive, then the operating system is called to change the current drive
*** to the search drive, the current working directory of that drive is
*** retrieved, and the current drive is reset to its original state.
***
*** If there is no search path in the command line, then the current
*** working directory is used as the search path.
**/

/* Get the current working directory for later */
getcwd(home_cwd_ptr, sizeof(path));

if ( (argc == 1) || (argv[argc-1][0] == '/') )
	search_ptr=home_cwd_ptr;
else {
	strcpy(search_ptr,argv[argc-1]);
	if ( (search_ptr[0] == home_cwd_ptr[0]) && (search_ptr[1] == ':') )
		search_ptr+=2;
	if (search_ptr[1] != ':')
		get_targetdir(search_ptr, home_cwd_ptr);
	else {
		home_drive[0]=home_cwd_ptr[0];
		search_drive[0]=search_ptr[0];
		if(system(search_drive) != 0) {
			puts(sys_errlist[errno]);
			return;
			}
		getcwd(alt_cwd_ptr, sizeof(path));
		if (search_ptr[2] == '\0')
			search_ptr=alt_cwd_ptr;
		get_targetdir(search_ptr, alt_cwd_ptr);
		system(home_drive);
		}
	}

/**
*** go for it
**/
if (force_lower)
	strlwr(search_ptr);
if (search_ptr[0])
	delve(search_ptr);
if (summary) {
	puti(dirs_found, "   "," directories found");
	puti(files_found, ", containing ", " files\n");
	}
return;
}



/***************************************************************************/
/***************************************************************************/
/**
*** delve -- function that recursively steps down the directory structure
*** of a disk, either printing directories/files or just watching them
*** go by, according to the command line flags.
**/
/***************************************************************************/
/***************************************************************************/

int delve( unsigned char *parent_ptr )

{
struct find_t   my_dta;
unsigned int    find_result;
path            current_path;
unsigned char   *current_end;


/**
*** Copy the path passed from above, and stick "\*.*" at the end of it.
*** current_end ends up pointing to the spot at which to stick in file
*** when propigate(sp?) names down the line (i.e. the 3rd char from the
*** right in current_path).
**/
strcpy(current_path, parent_ptr);
current_end=strchr(current_path,'\0');
if (*(current_end-1) != '\\')
	*(current_end++)='\\';
strcpy(current_end,"*.*");

/**
*** Find all the subdirectories in this directory.  If there are no files
*** in this directory at all, then this must not be a valid directory, so
*** the parent pathname is not printed.  The '.' and '..' directories
*** are duly ignored.
**/
find_result=_dos_findfirst(current_path, srch_attrib, &my_dta);
if (find_result == 0) {
	if (print_dirs)
		puts(parent_ptr+print_drive);
	++dirs_found;
	}

while (find_result == 0) {
	if ( my_dta.name[0] != '.' ) {
		namecpy(current_end, my_dta.name);
		if (force_lower)
			strlwr(current_end);
		if ( my_dta.attrib & _A_SUBDIR ) {
			delve(current_path);
			}
		else {
			++files_found;
			if (print_files)
				if (print_path)
					puts(current_path+print_drive);
				else
					puts(current_end);
			}
		}   /* if my_dta.name[0] != '.' */
	find_result=_dos_findnext(&my_dta);
	}
return(0);
}



/***************************************************************************/
/***************************************************************************/
/**
*** get_targetdir -- TRUNK has to find the target directory in more than
*** one place in the code, so this function consolidates them.  The target
*** directory passed to this function can be relative to the current
*** directory (such as '..'), so we chdir to that directory, ask DOS what the
*** current pathname is, and place that in *target.  Finally we change back
*** to the directroy whence we came.  If the target directory does not exist,
*** *target is set to a null string.
**/
/***************************************************************************/
/***************************************************************************/

char *get_targetdir(unsigned char *target, /* place to put target pathname */
					unsigned char *home     /* directory we just came from */
					)

{
if (chdir(target) != 0)
	target[0]='\0';
else {
	getcwd(target, sizeof(path));
	chdir(home);
	}
return(target);
}



/***************************************************************************/
/***************************************************************************/
/**
*** puti -- written to avoid using printf, which would add 2K to the EXE
*** file size.  The three arguments consist of the integer to print, a
*** pointer to a string to be printed before the integer, and a similar
*** pointer to be printed after the integer.
***
*** E.G.: puti(52,"There are "," cards in a deck.\n") yields
*** There are 52 cards in a deck.
**/
/***************************************************************************/
/***************************************************************************/

int puti(int the_int, char *pre_str, char *post_str)

{
char    number[7];
int     n=0;


itoa(the_int, number, 10);
while (*pre_str != '\0')
	putchar(*(pre_str++));
while (number[n] != '\0')
	putchar(number[n++]);
while (*post_str != '\0')
	putchar(*(post_str++));
return(0);
}



/***************************************************************************/
/***************************************************************************/
/**
*** namecpy -- function to take a filename from a dta--which is padded with
*** spaces--and turn it into a string, which is placed at *target_ptr.
**/
/***************************************************************************/
/***************************************************************************/

char *namecpy(  unsigned char *target_ptr,  /* place to put the filename */
				unsigned char *dta_name_ptr /* pointer into the DTA */
				)

{
unsigned char   *dest_ptr=target_ptr;
unsigned char   ch;


if (*dta_name_ptr != '\0')      /* properly handle *dta_name_ptr=='\0' */
	while ( (ch=*dta_name_ptr++) != '\0') {
		if (ch != ' ')
			*(dest_ptr++)=ch;
		}
*dest_ptr='\0';
return(target_ptr);
}



/***************************************************************************/
/***************************************************************************/
/**
*** display_strings -- use puts to print a series of strings to stdout until
*** a null string is encountered.
**/
/***************************************************************************/
/***************************************************************************/

void display_strings( char *msgs[] )
{

while ( msgs[0][0] )
	puts( msgs++[0] );
return;
}
